#ifndef PATCH_H
#define PATCH_H

#include <stdint.h>
#include <jack/types.h>

/* magic numbers */
enum { PATCH_POLYPHONY = 32,
       PATCH_COUNT = 64,
       PATCH_MAX_NAME = 64,
       PATCH_MAX_SAMPLE_NAME = 256 /* maximum sample file name length */
};

/* error codes */
enum { PATCH_ID_INVALID = -1,
       PATCH_EMPTY_LIST = -2,
       PATCH_ID_COLLISION = -3,
       PATCH_ALLOC_FAIL = -4,
       PATCH_SAMPLE_FAIL = -5,
       PATCH_NOTE_INVALID = -6,
       PATCH_FILE_WRITE_FAIL = -7,
       PATCH_FILE_OPEN_FAIL = -8,
       PATCH_PAN_INVALID = -9,
       PATCH_CHANNEL_INVALID = -10,
       PATCH_VOL_INVALID = -11,
       PATCH_MASTER_VOLUME_INVALID = -12,
       PATCH_PLAY_MODE_INVALID = -13,
       PATCH_LIMIT = -14,
       PATCH_SAMPLE_INDEX_INVALID = -15
};

typedef enum 
play_mode_s { 
     PM_NORMAL,
     PM_TRIM,
     PM_LOOP,
     PM_PINGPONG,
     PM_REVERSE
} play_mode_t;

/* patch management */
void patch_activate(int chan, int note, int vel);
void patch_activate_by_id(int id);
int patch_channel_get(int id);
int patch_channel_set(int id, int channel);
int patch_create(const char *name);
int patch_cut_by_get(int id);
int patch_cut_by_set(int id, int cut_by);
int patch_cut_get(int id);
int patch_cut_set(int id, int cut);
void patch_deactivate(int chan, int note);
void patch_deactivate_by_id(int id);
int patch_destroy(int id);
void patch_destroy_all();
int patch_display_index_get(int id);
int patch_dump(int **dump);
int patch_file_open(char *name);
int patch_file_write(char *name);
void patch_flush();
int patch_frames_get(int id);
int patch_loop_start_get(int id);
int patch_loop_start_set(int id, int start);
int patch_loop_stop_get(int id);
int patch_loop_stop_set(int id, int stop);
int patch_lower_note_get(int id);
int patch_lower_note_set(int id, int note);
int patch_master_volume_set(int val);
void patch_mixdown(float *buf, int frames);
int patch_name_set(int id, const char *name);
char *patch_name_get(int id);
int patch_note_get(int id);
int patch_note_set(int id, int note);
int patch_pan_get(int id);
int patch_pan_set(int id, int pan);
play_mode_t patch_play_mode_get(int id);
int patch_play_mode_set(int id, play_mode_t mode);
void patch_preview(char *name);
int patch_range_get(int id);
int patch_range_set(int id, int range);
int patch_sample_load(int id, char *file);
float const *patch_sample_get(int id);
char *patch_sample_name_get(int id);
void patch_sample_reload_all();
int patch_sample_start_get(int id);
int patch_sample_start_set(int id, int start);
int patch_sample_stop_get(int id);
int patch_sample_stop_set(int id, int stop);
void patch_sample_unload(int id);
const char *patch_strerror(int error);
int patch_upper_note_get(int id);
int patch_upper_note_set(int id, int note);
int patch_verify(int id);
int patch_volume_get(int id);
int patch_volume_set(int id, int vol);

#endif
